# -*- coding: utf-8 -*-
"""
Created on Fri Apr 14 13:37:03 2023

@author: User
"""

# adapted from https://github.com/facebookresearch/segment-anything
# licence Apache, https://github.com/facebookresearch/segment-anything/blob/main/LICENSE

import cv2
import numpy as np
from dl.machine_learning.segment_anything import sam_model_registry, SamPredictor, SamAutomaticMaskGenerator
from dl.utils.dl_downloads import get_sam

class SegmentAnything():
    def __init__(self):
        self.model ='vit_b'
        self.device = "cuda"
        self.initialized = False
        self.image = None

    def setModel(self,modelname):
        if modelname != self.model:
            self.model = modelname
            self.initialized = False
            self.image = None

        
    def loadModel(self):
        ckpt = get_sam(self.model)
        self.sam = sam_model_registry[self.model](checkpoint=ckpt)
        self.sam.to(device=self.device)

        self.predictor = SamPredictor(self.sam)
        self.mask_generator = SamAutomaticMaskGenerator(self.sam)
        self.initialized = True
        
        
    def setImage(self, image):
        try:
            if not self.initialized:
                self.loadModel()
                
            if self.image is None:
                self.predictor.set_image(image)
                self.image = image
            else:
                if self.image.shape != image.shape or not (self.image==image).all():
                    self.predictor.set_image(image)
                    self.image = image
        except:
            raise

    def SegmentImage(self, image):
        try:
            if not self.initialized:
                self.loadModel()
            mask_image = np.zeros((image.shape[0],image.shape[1]), dtype=np.uint8)
            masks = self.mask_generator.generate(image)
            contours = []
            for mask in masks:
                m = mask['segmentation']
                contour, _ = cv2.findContours(m.astype(np.uint8), cv2.RETR_EXTERNAL , cv2.CHAIN_APPROX_SIMPLE)
                contours.append(contour[0])
            
            for c in contours:
                cv2.drawContours(mask_image, [c], -1, (0), 3)  
                cv2.drawContours(mask_image, [c], -1, (1), -1) 

            return mask_image
        except:
            raise
        
    def getMask(self, bbox, points, labels):
        mask, _, _ = self.predictor.predict(
            point_coords=points,
            point_labels=labels,
            box=bbox,
            multimask_output=False,
            )
        return mask


# -*- coding: utf-8 -*-
"""
Created on Fri Jun 26 10:37:21 2020

@author: Koerber
"""


# import utils.Contour as Contour
# import utils.Point as Point

import glob
import os
from itertools import repeat, chain
from utils.Image import supportedImageFormats

from enum import Enum



class dlStackLabels(Enum):
    separated = 1
    unique_collectiveInput = 2
    unique_detachedInput = 3

def getMatchingImageLabelPairsRecursive(imagepath, labelfoldername, stacklabels=dlStackLabels.separated):
    images = []
    labels = []
    for root, dirs, files in os.walk(imagepath): 
        if labelfoldername in dirs:
            # this is inefficent because we search again content of subdir, which is the information we already have in files or dirs+files
            img, lab = getMatchingImageLabelPairPaths(root,os.path.join(root,labelfoldername),stacklabels)
            if img and lab:
                images.extend(img)
                labels.extend(lab)
                
    return images, labels

def getMatchingImageLabelPairPaths(imagepath, labelpath, stacklabels=dlStackLabels.separated, unroll=True):
    if not imagepath or not labelpath:
        return list(), list()
    if not os.path.isdir(imagepath) or not os.path.isdir(labelpath):
        return list(), list()


    images = glob.glob(os.path.join(imagepath,'*.*'))
    labels = glob.glob(os.path.join(labelpath,'*.*'))
              
    labels = [x for x in labels if x.endswith(".npz")]
    images = [x for x in images if x.lower().endswith(tuple(supportedImageFormats()))]
    
    image_names = [os.path.splitext(os.path.basename(each))[0] for each in images]
        
    # adds subfolders to labels as image stack labels are saved in folders
    labels.extend([os.path.join(labelpath, f) for f in os.listdir(labelpath) if (os.path.isdir(os.path.join(labelpath, f)) and f in ''.join(os.listdir(os.path.join(labelpath, f))))])
    label_names = [os.path.splitext(os.path.basename(each))[0] for each in labels]

    intersection = list(set(image_names).intersection(label_names))
    if intersection == list():
        return None, None
        

    images = [each for each in images if os.path.splitext(os.path.basename(each))[0] in intersection]
    labels = [each for each in labels if os.path.splitext(os.path.basename(each))[0] in intersection]
    images.sort()
    labels.sort()
    

    
    if unroll:
        return unrollPaths(images, labels, stacklabels)
    else:
        return images, labels

def splitStackLabels(image, folder, stacklabels):
    if not os.path.isdir(os.path.join(folder)):
        return [image], [folder]

    if stacklabels == dlStackLabels.unique_collectiveInput:
        labelname = getUniqueStackLabelFromFolderPath(folder)

        if labelname:
            label = (labelname, '-1')
            image = (image, dlStackLabels.unique_collectiveInput)
            return [image], [label]
        else:
            return [], []

    elif stacklabels == dlStackLabels.unique_detachedInput:
        labelname = getUniqueStackLabelFromFolderPath(folder)
        if labelname:
            maxframes = getFrameFromLabelPath(labelname, labeltype = stacklabels)
            images = list(repeat((image, dlStackLabels.separated), maxframes))
            labels = [(labelname,i) for i in range(maxframes)]
            return images, labels
        else:
            return [], []
        

    elif stacklabels == dlStackLabels.separated:
        subfiles = [(os.path.join(folder,f),f[-7:-4]) for f in os.listdir(folder) if os.path.splitext(os.path.basename(folder))[0]+f[-8:-4] in f]
        images = list(repeat((image, dlStackLabels.separated), len(subfiles)))
        return images, subfiles
    else:
        return [], []

def unrollPaths(images, labels, stacklabel):
    if not images or not labels:
        return None, None
    split = [splitStackLabels(x,y,stacklabel) for x,y in zip(images, labels)]
    images,labels = zip(*split)
    images = list(chain.from_iterable(images))
    labels = list(chain.from_iterable(labels))
    return images, labels

def getUniqueStackLabelFromFolderPath(folder):
    files = os.listdir(folder)
    labels = [f for f in files if (f.startswith(os.path.splitext(os.path.basename(folder))[0]) and f.endswith('u.npz'))]
    if len(labels) == 1:
        return os.path.join(folder, labels[0])
    else:
        return None
        

def CheckIfStackLabel(path):
    if isinstance(path,tuple):
        return True
    else:
        return False

def extendLabelNameByFrame(labelname, frame):
    return labelname + '_' + "{0:0=3d}".format(frame)

def extendLabelNameByMaxFrame(labelname, maxframe):
    return labelname + '_' + "{0:0=3d}".format(maxframe) + 'u'

def removeFrameFromLabelName(labelname, labeltype = dlStackLabels.separated):
    if  labeltype == dlStackLabels.separated:
        return labelname[:-4]
    else:
        return labelname[:-5]

def getFrameFromLabelPath(labelpath, labeltype = dlStackLabels.separated):
    if  labeltype == dlStackLabels.separated:
        return int(labelpath[-7:-4])
    else:
        return int(labelpath[-8:-5])


# -*- coding: utf-8 -*-
"""
Created on Fri Feb 18 16:38:21 2022

@author: Koerber
"""

from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from ui.style import styleForm
from ui.ui_utils import ScrollLabel
import os

class Window(object):
    def setupUi(self, Form):

        Form.setWindowTitle('Log File') 
        styleForm(Form)
        
        self.LogText = ScrollLabel(Form)
        self.setCentralWidget(self.LogText)
        self.LogText.setText('No log-file created, yet')
        
        self.resize(650,350)
        

class LogWindow(QMainWindow, Window):
    def __init__(self, parent ):
        super(LogWindow, self).__init__(parent)
        self.parent = parent
        self.setupUi(self)

    def showLog(self, logpath):
        if os.path.exists(logpath):
            f = open(logpath, "r")
            self.LogText.setText(f.read())
